const SVG = "http://www.w3.org/2000/svg";
const KEY_LEFT = 37;
const KEY_RIGHT = 39;
const CHAR_PLUS = 43;
const CHAR_MINUS = 45;

var gDoc, gXMLDoc;
var gNeedPreprocessing = false;
var gProdTreeItems = { };
var gPreviousSelectedNode = {
  elt: null,
  style: "",
  newStyle: "",
  modified: false
};

var COPYCOL = 2;
var gAttrView = new treeView(["attr-ns","attr-name","attr-value", "attr-idval"],
                             COPYCOL);

var ios = Components.classes["@mozilla.org/network/io-service;1"]
                    .getService(Components.interfaces.nsIIOService);

function ppAterm(term)
{
  var i = makeTempFile("ag-viewer-pp-aterm-in");
  var o = makeTempFile("ag-viewer-pp-aterm-out");
  writeToFile(i, term);
  runProgram("pp-aterm -i " + i.path + " -o " + o.path);
  var res = readFromFile(o);
  i.remove(false);
  o.remove(false);
  return res;
}

function setStatus(aMsg)
{
  var status = document.getElementById("status");
  status.label = aMsg;
}

function updateSelection()
{
  if (gPreviousSelectedNode.elt)
  {
    var style;
    if (gPreviousSelectedNode.modified)
    {
      style = gPreviousSelectedNode.style;
      gPreviousSelectedNode.modified = false;
    }
    else
    {
      style = gPreviousSelectedNode.newStyle;
      gPreviousSelectedNode.modified = true;
    }
    gPreviousSelectedNode.elt.setAttribute("style", style);
  }
  setTimeout(updateSelection, 500);
}

setTimeout(updateSelection, 1000);

function makeColor()
{
  preprocessColors(gDoc);
  setStatus("preprocessing done");
}

function zoom(ratio)
{
  var browser = document.getElementById("browser");
  gDoc.documentElement.currentScale *= ratio;
  if (browser.markupDocumentViewer.fullZoom)
    browser.markupDocumentViewer.fullZoom = gDoc.documentElement.currentScale;
}

function mouseIn(aEvent)
{
  if (aEvent.target.localName != "ellipse")
    return;

  var id = aEvent.target.getAttribute("id") || aEvent.target.parentNode.firstChild.textContent;
  var nsResolver = gXMLDoc.createNSResolver(gXMLDoc.documentElement);
  var exp = '//prod[@id="' + id + '"]/@name';
  var res = gXMLDoc.evaluate(exp, gXMLDoc, nsResolver,
                             XPathResult.STRING_TYPE, null);

  setStatus(res.stringValue);
}

function ensureOpen(aItem)
{
  var tree = document.getElementById("prodtree");
  var index = tree.view.getIndexOfItem(aItem);
  if (index == -1)
    ensureOpen(aItem.parentNode.parentNode);
  else
    tree.view.toggleOpenState(index);
}

function selectItem(aItem)
{
  var tree = document.getElementById("prodtree");
  var index = tree.view.getIndexOfItem(aItem);
  while (index == -1)
  {
    // the node that we want to select isn't visible in the productions tree.
    ensureOpen(aItem.parentNode.parentNode);
    index = tree.view.getIndexOfItem(aItem);
  }
  tree.treeBoxObject.ensureRowIsVisible(index);
  tree.view.selection.select(index);
}

function clickTree(aEvent)
{
  var elt = aEvent.target;
  if (elt.localName != "ellipse")
    return;

  var id = elt.getAttribute("id") || elt.parentNode.firstChild.textContent;
  var item = gProdTreeItems[id];
  selectItem(item);
  selectNode(elt);
 }

function onPageLoad(aEvent)
{
  gDoc = aEvent.originalTarget;
  // gDoc is the document that triggered "onload" event
  if (gNeedPreprocessing)
    doPreprocessing(gDoc);
  gDoc.addEventListener("mouseover", mouseIn, false);
  gDoc.addEventListener("click", clickTree, false);
  gDoc.addEventListener("mouseout", function() { setStatus(""); }, false);
}

function handleKeys(e)
{
  if (!e.keyCode)
  {
    if (e.charCode == CHAR_PLUS)
      zoom(2);
    else
      if (e.charCode == CHAR_MINUS)
        zoom(0.5);
  }
}

function onSelectProd()
{
  var tree = document.getElementById("prodtree");
  var id = tree.view.getCellText(tree.currentIndex,
                                 tree.columns.getNamedColumn("prod-id"));
  var nsResolver = gXMLDoc.createNSResolver(gXMLDoc.documentElement);
  var exp = '//prod[@id="' + id + '"]/attr';
  var resIterator = gXMLDoc.evaluate(exp, gXMLDoc, nsResolver,
                                     XPathResult.ANY_TYPE, null);
  gAttrView.clear();
  for (var attr = resIterator.iterateNext(); attr; attr = resIterator.iterateNext())
  {
    var idval = attr.getAttribute("idvalue");
    exp = '//val[@id="val_' + idval + '"]/@value';
    var res = gXMLDoc.evaluate(exp, gXMLDoc, nsResolver,
                               XPathResult.STRING_TYPE, null);

    gAttrView.addRow([attr.getAttribute("ns"),
                      attr.getAttribute("name"),
                      res.stringValue,
                      idval]);
  }

  var elt = gDoc.getElementById(id);
  if (elt)
    selectNode(elt);

  var cmdLine = window.arguments[0];
  var cmdLineArgument = cmdLine.getArgument(0);
  var file = cmdLine.resolveFile(cmdLineArgument);
  loadSrcView(file);

  var textbox = document.getElementById("src-display");
  if (!textbox.hidden)
  {
    var prod = gXMLDoc.evaluate('//prod[@id="' + id + '"]', gXMLDoc, nsResolver,
                                XPathResult.ANY_UNORDERED_NODE_TYPE,
                                null).singleNodeValue;
    var start = parseInt(prod.getAttribute("pos"));
    var end = start + parseInt(prod.getAttribute("length"));
    textbox.setSelectionRange(start, end);
    // if we don't explicitely give focus, the selection is not always visible.
    textbox.focus();
  }
}

function onClickProd(event)
{
  // if it's not a doubleclick we exit the function
  if (event.detail != 2)
    return;

  // get the index of the clicked row in the tree
  var tree = document.getElementById("prodtree");
  var row = {}, col = {}, obj = {};
  tree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, obj);

  // prevent the node from getting closed in the tree
  // (this is the default action of the doubleclick)
  event.stopPropagation();

  // get the id of the corresponding production
  var id = tree.view.getCellText(row.value, tree.columns.getNamedColumn("prod-id"));

  var elt = gDoc.getElementById(id);
  if (elt)
  {
    selectNode(elt);

    // scroll to make the selected node visible
    var bb = elt.getBBox();
    gDoc.defaultView.scrollTo(bb.x + bb.width / 2, bb.y + bb.height / 2);
  }
}

function onClickAttr(event)
{
  // if it's not a doubleclick we exit the function
  if (event.detail != 2)
    return;

  // get the index of the clicked row in the tree
  var tree = document.getElementById("attrtree");
  var row = {}, col = {}, obj = {};
  tree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, obj);

  // prevent the node from getting closed in the tree
  // (this is the default action of the doubleclick)
  event.stopPropagation();

  // get the value of the corresponding attribute
  var term = tree.view.getCellText(row.value, tree.columns.getNamedColumn("attr-value"));

  var textbox = document.getElementById("src-display");
  textbox.value = ppAterm(term);
  textbox.hidden = false;
}

function selectNode(elt)
{
  // restaure the default apparence of the previously selected node if any
  if (gPreviousSelectedNode.elt)
    gPreviousSelectedNode.elt.setAttribute("style", gPreviousSelectedNode.style);

  // save the current style of the node
  gPreviousSelectedNode.style = elt.getAttribute("style");
  gPreviousSelectedNode.elt = elt;
  gPreviousSelectedNode.newStyle = elt.getAttribute("style").replace(/fill: [a-z]+;/, "fill: blue;");
  gPreviousSelectedNode.newStyle += "stroke-width: 5";

  // highlight the selected node with a big border
  elt.setAttribute("style", gPreviousSelectedNode.newStyle);
  gPreviousSelectedNode.modified = true;
}

function newTreeItem(aValues)
{
  var treeitem = document.createElement("treeitem");
  var treerow = document.createElement("treerow");
  for (var i = 0; i < aValues.length; ++i) {
    var treecell = document.createElement("treecell");
    treecell.setAttribute("label", aValues[i]);
    treerow.appendChild(treecell);
  }
  treeitem.appendChild(treerow);
  return treeitem;
}

function addProd(aParent, aProd)
{
  var name = aProd.localName == "prod" ? aProd.getAttribute("name")
                                       : aProd.localName;
  var id = aProd.getAttribute("id");
  var treeitem = newTreeItem([name, id])
  gProdTreeItems[id] = treeitem;
  aParent.appendChild(treeitem);

  var nextNode = aProd.firstChild;
  while (nextNode && nextNode.localName != "prod"
                  && nextNode.localName != "amb")
    nextNode = nextNode.nextSibling;

  if (nextNode) {
    treeitem.setAttribute("container", "true");
    treeitem.setAttribute("open","true");
    var treechildren = document.createElement("treechildren");
    treeitem.appendChild(treechildren);
    addProd(treechildren, nextNode);
    while (nextNode)
    {
      nextNode = nextNode.nextSibling;
      if (nextNode && (nextNode.localName == "prod" ||
                       nextNode.localName == "amb"))
        addProd(treechildren, nextNode);
    }
  }
}

function makeTree(url)
{
  var req = new XMLHttpRequest();
  req.open("GET", url, false);
  req.send(null);

  // a status of 0 is expected for file:// or ftp://, 200 is expected for http
  if (req.status && req.status != 200)
  {
    alert(req.status + " error code received");
    return;
  }
  gXMLDoc = req.responseXML;

  var firstParent = document.getElementById("first-child");
  var program = gXMLDoc.getElementsByTagName("program");
  if (!program.length)
    return false;
  addProd(firstParent, program[0].firstChild);

  document.getElementById("prodtree").addEventListener("click", onClickProd, true);
  document.getElementById("attrtree").addEventListener("click", onClickAttr, true);

  return true;
}

function loadSVGFile(aURI)
{
  var url = aURI.spec;
  var browser = document.getElementById("browser");
  browser.loadURI(url, null, null);

  var xmlfile = url.replace(/\.svg$/, ".xml");
  if (!makeTree(xmlfile))
  {
    document.getElementById("left-part").collapsed = true;
    gNeedPreprocessing = true;
  }
}

function loadSrcView(aFile)
{
  var src = makeTempFile("ag-viewer.src");
  runProgram("asfix-yield -i " + aFile.path + " -o " + src.path);

  var res = readFromFile(src);
  if (res)
  {
    var textbox = document.getElementById("src-display");
    textbox.value = res;
    textbox.hidden = false;
  }
}

function createSVGandXMLfiles(aFile)
{
  var svg = makeTempFile("ag-viewer-attr-2-dot.svg");
  var xml = makeTempFile("ag-viewer-attr-2-dot.xml");

  runProgram("attr-2-dot -s -i " + aFile.path + " |pp-dot |dot -Tsvg |simplify-dot-svg > " + svg.path);
  runProgram("attr-2-dot -x -i " + aFile.path + " --depth 20 |pp-xml-info -o " + xml.path);

  return ios.newFileURI(svg);
}

function onload()
{
  var browser = document.getElementById("browser");
  browser.addEventListener("load", onPageLoad, true);
  browser.addEventListener("keypress", handleKeys, false);

  var attrTree = document.getElementById("attrtree");
  attrTree.view = gAttrView;

  var cmdLine = window.arguments[0];
  var cmdLineArgument = cmdLine.getArgument(0);
  document.title = cmdLineArgument + " - AG-Viewer";
  if (/\.svg$/.test(cmdLineArgument))
  {
    // We have the url of an SVG file
    var url = cmdLine.resolveURI(cmdLineArgument);
    loadSVGFile(url);
  }
  else
  {
    // We have the path to a file, maybe aterm
    var wrongParam = false;
    try
    {
      var file = cmdLine.resolveFile(cmdLineArgument);
    }
    catch(e)
    {
      // The file does not exist
      wrongParam = true;
    }
    if (!cmdLineArgument || wrongParam)
      browser.loadURI("data:text/html,<p>Please give as command line argument either a path or an url to an svg or aterm file</p>", null, null);
    else
    {
      var url = createSVGandXMLfiles(file);
      if (url)
      {
	loadSVGFile(url);
	loadSrcView(file);
      }
      else
	browser.loadURI("data:text/html,<p>Wrong file format.</p>");
    }
  }
}
