#include <srts/stratego.h>
#include <stratego-lib/stratego-lib-common.h>
#include "stable.h"

static ATerm main_table;
static Symbol hashtable_sym;

struct free_list_s {
  struct free_list_s *next;
  stable_blob_t *blob;
};

struct free_list_s *free_list;

static void init_module_constant_terms(void)
{
  ATerm table = hashtable_to_term(ATtableCreate(1024, 80));
  ATprotect(&(main_table));
  main_table = (ATerm) ATmakeAppl(hashtable_sym, table);
  free_list = NULL;
}

static
ATbool stable_blob_destructor(ATermBlob);

static void init_module_constructors(void)
{
  hashtable_sym = ATmakeSymbol("Hashtable", 1, ATfalse);
  ATprotectSymbol(hashtable_sym);

  ATregisterBlobDestructor(stable_blob_destructor);
}

#include <srts/init-stratego-module.h>

#define MAGIC 0x13795a5a

struct str_frame fake_frame = { .parent = &fake_frame,
				.vars = NULL,
				.funs = NULL};

static
ATbool stable_blob_destructor(ATermBlob b)
{
  stable_blob_t *v;

  if (ATgetBlobSize(b) != sizeof (stable_blob_t))
    return ATfalse;
  v = ATgetBlobData(b);
  if (v->magic != MAGIC)
    return ATfalse;

  struct free_list_s *old = free_list;
  free_list = malloc(sizeof(struct free_list_s));
  if (free_list == NULL) {
    ATfprintf(stderr, "Failure while allocating a free_list_s element\n");
    free_list = old;
  }
  free_list->next = old;
  free_list->blob = v;

  return ATtrue;
}

ATerm STABLE_free_pending()
{
  while (free_list != NULL) {
    stable_blob_t *v = free_list->blob;
    struct free_list_s *old = free_list;
    free_list = free_list->next;
    free(old);
    v->strategy(&fake_frame, v->input);
    ATunprotect(&(v->input));
    free(v);
  }

  return (ATerm) ATmakeInt(0);
}

ATerm STABLE_register_destructor(StrCL s, ATerm input)
{

  stable_blob_t* blob = calloc(1, sizeof (stable_blob_t));

  if (blob == NULL)
  {
    ATfprintf(stderr, "Failure while allocating a blob.\n");
    _fail(NULL);
  }

  ATprotect(&(blob->input));

  blob->magic = MAGIC;
  blob->strategy = cl_fun(s);
  //  blob->static_link = cl_sl(s);
  blob->input = input;

  return (ATerm) ATmakeBlob(sizeof (stable_blob_t), blob);
}

ATerm STABLE_new_index()
{
  static int current = 0;

  return (ATerm) ATmakeInt(current++);
}

ATerm STABLE_main_table()
{
  return main_table;
}
